package org.richfaces.renderkit.html;

import org.ajax4jsf.javascript.JSFunction;
import org.ajax4jsf.javascript.JSFunctionDefinition;
import org.ajax4jsf.javascript.JSObject;
import org.ajax4jsf.javascript.JSReference;
import org.ajax4jsf.renderkit.AjaxRendererUtils;
import org.ajax4jsf.renderkit.HeaderResourcesRendererBase;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.richfaces.component.UICoordinatesGrid;
import org.richfaces.component.event.CoordinatesRefreshEvent;
import org.richfaces.component.event.CoordinatesSelectionEvent;

import javax.faces.component.NamingContainer;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

public class CoordinatesGridRendererBase extends HeaderResourcesRendererBase {

    private static Log logger = LogFactory.getLog(CoordinatesGridRendererBase.class);
    private static final String EVENT_TYPE_PARAM = "eventType";
    private static final String START_X_PARAM = "startX";
    private static final String END_X_PARAM = "endX";
    private static final String START_Y_PARAM = "startY";
    private static final String END_Y_PARAM = "endY";
    private static final String CALLBACK = "callback";
    private static final String SELECT_EVENT = "select";
    private static final String REFRESH_EVENT = "refresh";
    public static final String AJAX_MODE = "ajax";
    public static final String SERVER_MODE = "server";
    public static final String CLIENT_MODE = "client";

    protected Class<? extends UIComponent> getComponentClass() {
        return UICoordinatesGrid.class;
    }

    @Override
    protected void doDecode(FacesContext context, UIComponent component) {
        super.doDecode(context, component);
        UICoordinatesGrid uiCoordinatesGrid;

        if (component instanceof UICoordinatesGrid) {
            uiCoordinatesGrid = (UICoordinatesGrid) component;
        } else {
            if (logger.isDebugEnabled()) {
                logger.debug("No decoding necessary since the component " + component.getId() +
                    " is not an instance or a sub class of UIInplaceInput");
            }
            return;
        }
        String startXParam = getFieldId(context, uiCoordinatesGrid, START_X_PARAM);
        String startYParam = getFieldId(context, uiCoordinatesGrid, START_Y_PARAM);
        String endXParam = getFieldId(context, uiCoordinatesGrid, END_X_PARAM);
        String endYParam = getFieldId(context, uiCoordinatesGrid, END_Y_PARAM);
        String eventTypeParam = getFieldId(context, uiCoordinatesGrid, EVENT_TYPE_PARAM);

        Map<String, String> request = context.getExternalContext().getRequestParameterMap();

        String startXString = request.get(startXParam);
        String endXString = request.get(endXParam);
        String startYString = request.get(startYParam);
        String endYString = request.get(endYParam);
        String eventType = request.get(eventTypeParam);
        if ((uiCoordinatesGrid.isSelectable() && SELECT_EVENT.equals(eventType)) || REFRESH_EVENT.equals(eventType) && startXString != null && !"".equals(startXString)) {
            try {
                int startX = Integer.parseInt(startXString);
                int startY = Integer.parseInt(startYString);
                int endX = Integer.parseInt(endXString);
                int endY = Integer.parseInt(endYString);
                if (SELECT_EVENT.equals(eventType)) {
                    new CoordinatesSelectionEvent(startX, startY, endX, endY, uiCoordinatesGrid).queue();
                } else {
                    uiCoordinatesGrid.queueEvent(new CoordinatesRefreshEvent(startX, startY, endX, endY, uiCoordinatesGrid));
                }
            } catch (Exception e) {
                logger.error("Problem during decoding event for component " + component.getId() + " (params: " + startXParam + "=" + startXString + ";" + startYParam + "=" + startYString + ";" + endXParam + "=" + endXString + ";" + endYParam + "=" + endYString + ")");
            }
        }
    }

    protected String getFieldId(FacesContext context, UIComponent component, String attribute) {
        return getUtils().clientId(context, component) + NamingContainer.SEPARATOR_CHAR + attribute;
    }

    public void writeInitFunction(FacesContext context, UICoordinatesGrid component) throws IOException {
        ResponseWriter writer = context.getResponseWriter();
        if (component.getWidgetVar() != null) {
            writer.write("var " + component.getWidgetVar() + "=");
        }
        Map<String, Object> options = new HashMap<String, Object>();
        addOptionIfSet("onclick", component.getOnclick(), options);
        addOptionIfSet("onbeforeselect", component.getOnbeforeselect(), options);
        addOptionIfSet("onselect", component.getOnselect(), options);
        addOptionIfSet("rows", component.getRows(), options);
        addOptionIfSet("cols", component.getCols(), options);
        addOptionIfSet("selectable", component.isSelectable(), options);
        addOptionIfSet("colHeaderPosition", component.getColHeaderPosition(), options);
        addOptionIfSet("rowHeaderPosition", component.getRowHeaderPosition(), options);
        addOptionIfSet("data", component.getData(0, 0, component.getCols() - 1, component.getRows() - 1, true), options);

        writer.writeText(new JSObject("RichFaces.CoordinatesGrid",
            component.getClientId(context),
            options,
            createSubmitEventFunction(context, component)).toScript(), null);
    }

    protected Object createSubmitEventFunction(FacesContext context, UICoordinatesGrid component) {
        JSFunction jsFunction;
        Map<String, Object> params = new HashMap<String, Object>();
        params.put(getFieldId(context, component, EVENT_TYPE_PARAM), new JSReference(EVENT_TYPE_PARAM));
        params.put(getFieldId(context, component, START_X_PARAM), new JSReference(START_X_PARAM));
        params.put(getFieldId(context, component, START_Y_PARAM), new JSReference(START_Y_PARAM));
        params.put(getFieldId(context, component, END_X_PARAM), new JSReference(END_X_PARAM));
        params.put(getFieldId(context, component, END_Y_PARAM), new JSReference(END_Y_PARAM));
        if (isAjaxMode(component)) {
            jsFunction = AjaxRendererUtils.buildAjaxFunction(component, context);
            Map<String, Object> eventOptions = AjaxRendererUtils.buildEventOptions(context, component, params);
            eventOptions.put("oncomplete", new JSReference(CALLBACK));
            jsFunction.addParameter(eventOptions);
        } else if (SERVER_MODE.equals(component.getSwitchType())) {
            jsFunction = new JSFunction("Richfaces.jsFormSubmit",
                component.getClientId(context),
                getUtils().getNestingForm(context, component).getClientId(context),
                null,
                params);
        } else {
            return null;
        }
        return new JSFunctionDefinition("event", EVENT_TYPE_PARAM, START_X_PARAM, START_Y_PARAM, END_X_PARAM, END_Y_PARAM, CALLBACK).addToBody(jsFunction);
    }

    private void addOptionIfSet(String optionName, Object value, Map<String, Object> options) {
        if (value != null && value != "") {
            options.put(optionName, value);
        }
    }

    private boolean isAjaxMode(UICoordinatesGrid component) {
        String mode = component.getSwitchType();
        return AJAX_MODE.equals(mode) || "".equals(mode) || null == mode;
    }
}
