/*
 * JBoss, Home of Professional Open Source
 * Copyright , Red Hat, Inc. and individual contributors
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.richfaces.renderkit.html;

import org.ajax4jsf.javascript.JSFunction;
import org.ajax4jsf.renderkit.HeaderResourcesRendererBase;
import org.ajax4jsf.renderkit.RendererUtils.HTML;
import org.ajax4jsf.resource.InternetResource;
import org.richfaces.component.UIDock;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import java.io.IOException;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

public class DockRenderer extends HeaderResourcesRendererBase {
// ------------------------------ FIELDS ------------------------------

    private static final Map<String, Object> DEFAULTS;

    private final InternetResource[] scripts = {
        getResource("/org/richfaces/renderkit/html/scripts/jquery/jquery.js"),
        getResource("/org/richfaces/renderkit/html/scripts/jquery.jdock.js"),
        getResource("/org/richfaces/renderkit/html/scripts/richfaces.dock.js")
    };
    private final InternetResource[] styles = {
        getResource("/org/richfaces/renderkit/html/css/dock.xcss")
    };

// -------------------------- STATIC METHODS --------------------------

    static {
        Map<String, Object> defaults = new HashMap<String, Object>();
        defaults.put("active", -1);
        defaults.put("align", "bottom");
        defaults.put("coefficient", 1.5);
        defaults.put("duration", 300);
        defaults.put("fadeIn", 0);
        defaults.put("fadeLayer", "");
        defaults.put("flow", false);
        defaults.put("idle", 0);
        defaults.put("inactivity", 0);
        defaults.put("labels", "false");
        defaults.put("loader", "");
        defaults.put("noBuffer", false);
        defaults.put("size", 48);
        defaults.put("step", 50);
        DEFAULTS = Collections.unmodifiableMap(defaults);
    }

// --------------------- GETTER / SETTER METHODS ---------------------

    protected Class<? extends UIComponent> getComponentClass() {
        return UIDock.class;
    }

    @Override
    protected InternetResource[] getScripts() {
        return scripts;
    }

    @Override
    protected InternetResource[] getStyles() {
        return styles;
    }

// -------------------------- OTHER METHODS --------------------------

    @Override
    protected void doEncodeBegin(ResponseWriter writer, FacesContext context, UIComponent component) throws IOException {
        writer.startElement(HTML.DIV_ELEM, component);
        final String clientId = getUtils().clientId(context, component);
        writer.writeAttribute(HTML.id_ATTRIBUTE, clientId, HTML.id_ATTRIBUTE);
        writer.startElement(HTML.DIV_ELEM, component);
        writer.writeAttribute(HTML.id_ATTRIBUTE, getContainerClientId(context, component), HTML.id_ATTRIBUTE);
        Object styleClass = component.getAttributes().get("styleClass");
        styleClass = styleClass == null ? "" : " " + styleClass.toString();
        writer.writeAttribute(HTML.class_ATTRIBUTE, "rf-dk" + styleClass, HTML.class_ATTRIBUTE);
        Object style = component.getAttributes().get("style");
        style = style == null ? "" : style.toString();
        writer.writeAttribute(HTML.style_ATTRIBUTE, "display:none;" + style, HTML.style_ATTRIBUTE);
    }

    @Override
    protected void doEncodeEnd(ResponseWriter writer, FacesContext context, UIComponent component) throws IOException {
        writer.endElement(HTML.DIV_ELEM);
        writer.startElement(HTML.SCRIPT_ELEM, null);
        writer.writeAttribute(HTML.TYPE_ATTR, "text/javascript", "type");
        writer.writeText(new JSFunction("RichFaces.Dock", getContainerClientId(context, component),
            getOptions((UIDock) component)), null);
        writer.writeText(";", null);
        writer.endElement(HTML.SCRIPT_ELEM);
        writer.endElement(HTML.DIV_ELEM);
    }

    private String getContainerClientId(FacesContext context, UIComponent component) {
        return getUtils().clientId(context, component) + "Container";
    }

    protected Map<String, Object> getOptions(UIDock dock) throws IOException {
        /**
         * Include only attributes that are actually set.
         */
        Map<String, Object> options = new HashMap<String, Object>();
        addOptionIfSetAndNotDefault("active", dock.getActive(), options);
        addOptionIfSetAndNotDefault("align", dock.getAlign(), options);
        addOptionIfSetAndNotDefault("coefficient", dock.getCoefficient(), options);
        addOptionIfSetAndNotDefault("duration", dock.getDuration(), options);
        addOptionIfSetAndNotDefault("fadeIn", dock.getFadeIn(), options);
        addOptionIfSetAndNotDefault("fadeLayer", dock.getFadeLayer(), options);
        addOptionIfSetAndNotDefault("flow", dock.isFlow(), options);
        addOptionIfSetAndNotDefault("idle", dock.getIdle(), options);
        addOptionIfSetAndNotDefault("inactivity", dock.getInactivity(), options);
        addOptionIfSetAndNotDefault("labels", dock.getLabels(), options);
        addOptionIfSetAndNotDefault("loader", dock.getLoader(), options);
        addOptionIfSetAndNotDefault("noBuffer", dock.isNoBuffer(), options);
        addOptionIfSetAndNotDefault("size", dock.getSize(), options);
        addOptionIfSetAndNotDefault("step", dock.getStep(), options);
        return options;
    }

    protected void addOptionIfSetAndNotDefault(String optionName, Object value, Map<String, Object> options) {
        if (value != null && !"".equals(value)
            && !value.equals(DEFAULTS.get(optionName))
            && !(value instanceof Collection && ((Collection) value).size() == 0)
            && !(value instanceof Map && ((Map) value).size() == 0)) {
            options.put(optionName, value);
        }
    }
}
