/*
 * JBoss, Home of Professional Open Source
 * Copyright , Red Hat, Inc. and individual contributors
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */

package org.richfaces.renderkit.html;

import org.ajax4jsf.javascript.JSFunction;
import org.ajax4jsf.renderkit.HeaderResourcesRendererBase;
import org.ajax4jsf.renderkit.RendererUtils.HTML;
import org.ajax4jsf.resource.InternetResource;
import org.richfaces.component.UINotify;
import org.richfaces.component.UINotifyStack;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.context.ResponseWriter;
import java.io.IOException;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

public class NotifyRenderer extends HeaderResourcesRendererBase {

    private static final Map<String, Object> DEFAULTS;

    static {
        Map<String, Object> defaults = new HashMap<String, Object>();
        defaults.put("styleClass", "");
        defaults.put("nonblocking", false);
        defaults.put("nonblockingOpacity", UINotify.DEFAULT_NONBLOCKING_OPACITY);
        defaults.put("showHistory", true);
        defaults.put("animationSpeed", "slow");
        defaults.put("opacity", 1);
        defaults.put("showShadow", false);
        defaults.put("showCloseButton", true);
        defaults.put("appearAnimation", "fade");
        defaults.put("hideAnimation", "fade");
        defaults.put("sticky", false);
        defaults.put("stayTime", 8000);
        defaults.put("delay", 0);
        DEFAULTS = Collections.unmodifiableMap(defaults);
    }

    private final InternetResource[] scripts = {
        getResource("/org/richfaces/renderkit/html/scripts/jquery/jquery.js"),
        getResource("/org/richfaces/renderkit/html/scripts/jquery.pnotify.js"),
        getResource("/org/richfaces/renderkit/html/scripts/richfaces.notify.js")
    };
    private final InternetResource[] styles = {
        getResource("/org/richfaces/renderkit/html/css/jquery.pnotify.xcss")
    };

    @Override
    public void encodeEnd(FacesContext context, UIComponent component) throws IOException {
        if (!(component instanceof UINotify)) {
            return;
        }
        ResponseWriter writer = context.getResponseWriter();
        writer.startElement(HTML.DIV_ELEM, null);
        writer.writeAttribute(HTML.id_ATTRIBUTE, getUtils().clientId(context, component), "type");
        writer.startElement(HTML.SCRIPT_ELEM, null);
        writer.writeAttribute(HTML.TYPE_ATTR, "text/javascript", "type");
        writer.writeText(new JSFunction("RichFaces.Notify", getOptions(context, (UINotify) component)), null);
        writer.writeText(";", null);
        writer.endElement(HTML.SCRIPT_ELEM);
        writer.endElement(HTML.DIV_ELEM);
    }

    protected Map<String, Object> getOptions(FacesContext context, UINotify notify) throws IOException {
        /**
         * Include only attributes that are actually set.
         */
        Map<String, Object> options = new HashMap<String, Object>();
        addOptionIfSetAndNotDefault("title", notify.getTitle(), options);
        addOptionIfSetAndNotDefault("text", notify.getText(), options);
        addOptionIfSetAndNotDefault("sticky", notify.isSticky(), options);
        addOptionIfSetAndNotDefault("stayTime", notify.getStayTime(), options);
        Map<String, Object> animationOptions = new HashMap<String, Object>();
        addOptionIfSetAndNotDefault("appearAnimation", notify.getAppearAnimation(), animationOptions);
        addOptionIfSetAndNotDefault("hideAnimation", notify.getHideAnimation(), animationOptions);
        addOptionIfSetAndNotDefault("animation", animationOptions, options);
        addOptionIfSetAndNotDefault("animationSpeed", notify.getAnimationSpeed(), options);
        addOptionIfSetAndNotDefault("nonblocking", notify.isNonblocking(), options);
        addOptionIfSetAndNotDefault("nonblockingOpacity", notify.getNonblockingOpacity(), options);
        addOptionIfSetAndNotDefault("showHistory", notify.isShowHistory(), options);
        addOptionIfSetAndNotDefault("showShadow", notify.isShowShadow(), options);
        addOptionIfSetAndNotDefault("showCloseButton", notify.isShowCloseButton(), options);
        UINotifyStack stack = getStackComponent(context, notify);
        if (stack != null) {
            addOptionIfSetAndNotDefault("stack", getUtils().clientId(context, stack), options);
        }
        String styleClass = notify.getStyleClass();
        if (styleClass == null) {
            styleClass = "";
        }
        addOptionIfSetAndNotDefault("styleClass", getStackStyleClass(context, notify) + " " + styleClass, options);
        addOptionIfSetAndNotDefault("delay", notify.getDelay(), options);
        return options;
    }

    protected String getStackStyleClass(FacesContext context, UINotify notify) {
        UINotifyStack stack = getStackComponent(context, notify);
        return stack == null ? "" : stack.getStyleClass();
    }

    protected void addOptionIfSetAndNotDefault(String optionName, Object value, Map<String, Object> options) {
        if (value != null && !"".equals(value)
            && !value.equals(DEFAULTS.get(optionName))
            && !(value instanceof Collection && ((Collection) value).size() == 0)
            && !(value instanceof Map && ((Map) value).size() == 0)) {
            options.put(optionName, value);
        }
    }

    protected Class<? extends UIComponent> getComponentClass() {
        return UINotify.class;
    }

    @Override
    protected InternetResource[] getScripts() {
        return scripts;
    }

    @Override
    protected InternetResource[] getStyles() {
        return styles;
    }

    protected UINotifyStack getStackComponent(FacesContext context, UINotify notify) {
        String stackId = notify.getStack();
        if (stackId == null) {
            UIComponent parent = notify.getParent();
            while (parent != null && !(parent instanceof UINotifyStack)) {
                parent = parent.getParent();
            }
            return (UINotifyStack) parent;
        } else {
            UIComponent componentFor = getUtils().findComponentFor(context.getViewRoot(), stackId);
            if (componentFor instanceof UINotifyStack) {
                return (UINotifyStack) componentFor;
            } else {
                return null;
            }
        }
    }
}
